#include <regex.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "landerctl.h"

static const char cfg_line_regex_expr[] = "^([^ ]+) *= *([^ ]+)$";

landerctl_cfg_err landerctl_cfg_parse(landerctl_cfg *out, const char *path) {
  FILE *f = fopen(path, "r");

  if (f == NULL) {
    return landerctl_cfg_err_not_found;
  }

  regex_t cfg_line_regex;
  regcomp(&cfg_line_regex, cfg_line_regex_expr, REG_EXTENDED);

  // Accept lines of at most 256 lines
  char line[256];
  landerctl_cfg_err res = landerctl_cfg_err_incomplete;

  while (fgets(line, sizeof(line), f) != NULL) {
    // Last character might be a newline
    size_t len = strlen(line);

    if (line[len - 1] == '\n') {
      line[len - 1] = '\0';
    }

    regmatch_t reg_groups[3];

    if (regexec(&cfg_line_regex, line, 3, reg_groups, 0) != 0) {
      res = landerctl_cfg_err_not_found;

      break;
    }

    // api_key is currently the only value we parse
    int key_len = reg_groups[1].rm_eo - reg_groups[1].rm_so;

    if ((strlen("api_key") == key_len) &&
        (strncmp("api_key", &line[reg_groups[1].rm_so], key_len) == 0)) {
      int val_len = reg_groups[2].rm_eo - reg_groups[2].rm_so;
      char *buf = malloc(val_len + 1);
      strncpy(buf, &line[reg_groups[2].rm_so], val_len);

      out->api_key = buf;

      res = landerctl_cfg_err_ok;
      break;
    }
  }

  fclose(f);

  return res;
}
